DEPRECATED_CONNECTOR_NAMES = [
    'Azure Log Analytics',
    'ConsenSys Ethereum',
    'Corda Blockchain',
    'D&B Optimizer',
    'Dynamic 365',
    'Elastic OCR',
    'Ethereum Blockchain',
    'Excel',
    'Language Tool',
    'LinkedIn',
    'Microsoft Dataverse (Legacy)',
    'Microsoft Translator',
    'Module Q',
    'Office 365 Video',
    'Outlook Tasks',
    'Pitney Bowes Data Validation',
    'Pitney Bowes Tax Calculator',
    'Projectwise Share',
    'Recorded Future',
    'SFTP',
    'Skype for Business Online',
    'Todoist',
    'Xpertdoc',
]
